use std::fmt::Display;

use pango::{
    prelude::{FontExt, FontMapExt},
    Font as PangoFont, FontDescription, Language, SCALE,
};
use pangocairo::FontMap;

pub struct Font {
    font: PangoFont,
    language: Language,
}

impl Font {
    pub fn new(font: PangoFont) -> Self {
        Self {
            font,
            language: Language::default(),
        }
    }

    pub fn with_language(self, language: Language) -> Self {
        Self { language, ..self }
    }

    pub fn family(&self) -> String {
        self.font.describe().family().unwrap().to_string()
    }

    pub fn size(&self) -> i32 {
        self.font.describe().size() / SCALE
    }

    pub fn height(&self) -> i32 {
        self.font.metrics(Some(&self.language)).height() / SCALE
    }

    pub fn load_from_string(family: &str, size: usize) -> Option<Font> {
        let font_str = format!("{family} {size}");
        let fontmap = FontMap::default();
        let context = fontmap.create_context();
        let desc = FontDescription::from_string(&font_str);
        fontmap.load_font(&context, &desc).map(Font::new)
    }
}

impl Default for Font {
    fn default() -> Self {
        Font::load_from_string("monospace", 10).unwrap()
    }
}

impl Display for Font {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "\"{}\" {}", self.family(), self.size())
    }
}
